{***************************************************************************
 *   Copyright (C) 2008-2013 by Bernhard Gross                             *
 *   B.Gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        * 
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_about;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Buttons, ExtCtrls, process, FileUtil, u_exchange;

type

  { TF_about }

  TF_about = class(TForm)
    btn_close: TBitBtn;
    btn_homep: TBitBtn;
    btn_mail: TBitBtn;
    Image1: TImage;
    Memo1: TMemo;
    procedure btn_mailClick(Sender: TObject);
    procedure btn_homepClick(Sender: TObject);
    procedure FindDefaultBrowser(var Browser: string);
    procedure FormShow(Sender: TObject);
    

    
  private
    FDefaultBrowser: string;
    pdir:string;
    function Find_Exec(const ShortFilename: string): boolean;
    { private declarations }
  public
    { public declarations }
  end; 

var
  F_about: TF_about;

implementation

{ TF_about }


function TF_about.Find_Exec (const ShortFilename: string): boolean;
var Filename: String;
begin
 Filename:=SearchFileInPath(ShortFilename+GetExeExt,'',SysUtils.GetEnvironmentVariable('PATH'),PathSeparator,[sffDontSearchInBasePath]);
 Result:=Filename<>'';
 if Result then begin
   FDefaultBrowser:=Filename;
 end;
end;

procedure TF_about.FindDefaultBrowser(var Browser: string);
begin
  FDefaultBrowser:='';
  if FDefaultBrowser='' then begin
    if Find_Exec('konqueror')
    or Find_Exec('galeon')
    or Find_Exec('htmlview')
    or Find_Exec('firefox')
    or Find_Exec('mozilla')
    or Find_Exec('safari')
    {$IFDEF DARWIN}
    { safari is not on path for MacOS }
    or Find_Exec('/Applications/Safari.app/Contents/MacOS/Safari')
    {$ENDIF}
    or Find_Exec('netscape')
    or Find_Exec('opera')
    or Find_Exec('iexplore') then ;
  end;
  {$IFDEF WINDOWS}
  if FDefaultBrowser='' then begin
    If FileExists(pdir+'\Mozilla Firefox\firefox.exe') then FDefaultBrowser:=pdir+'\Mozilla Firefox\firefox.exe'
    else If FileExists(pdir+'\Internet Explorer\iexplore.exe') then FDefaultBrowser:=pdir+'\Internet Explorer\iexplore.exe'
  end;
 {$ENDIF}
  Browser:=FDefaultBrowser;
end;

procedure TF_about.FormShow(Sender: TObject);
begin
 Memo1.Lines[1]:=progvers;
 FDefaultBrowser:='';
 pdir:='';
 {$IFDEF WINDOWS}
  pdir:=GetEnvironmentVariable('ProgramFiles');
 {$ENDIF}
end;

procedure TF_about.btn_homepClick(Sender: TObject);
const url:string='www.pimicos.com';
var AProcess: TProcess;
    dbrowser:string;
begin
  dbrowser:='';
  FindDefaultBrowser(dbrowser);
  if dbrowser<>'' then begin
   AProcess:= TProcess.Create(nil);
   AProcess.CommandLine:= dbrowser+' '+url;
   AProcess.Execute;
   AProcess.Free;
   end
  else showmessage ('Can not locate any browser!');
end;

procedure TF_about.btn_mailClick(Sender: TObject);
var AProcess: TProcess;
    s,pf:string;
begin
  s:=Memo1.Lines[1];
  s:='Venus1Term'+'%20'+StringReplace(s,' ','%20',[ rfReplaceAll])+'%20';
  AProcess:= TProcess.Create(nil);
   AProcess.CommandLine:='';
  {$IFDEF LINUX}
   pf:= GetEnvironmentVariable('OSTYPE')+'-'+ GetEnvironmentVariable('HOSTTYPE');
   if  Find_Exec('kfmclient') then begin
    AProcess.CommandLine:='kfmclient exec mailto:B.Gross@pimicos.com?&subject='+s+pf;
   end
   else if Find_Exec('gnome-open') then begin
    AProcess.CommandLine:='gnome-open mailto:B.Gross@pimicos.com?subject='+s+pf;
   end;
  {$ENDIF}
  {$IFDEF WINDOWS}
   pf:= GetEnvironmentVariable('OS')+'-'+ GetEnvironmentVariable('PROCESSOR_ARCHITECTURE');
   if FileExists(pdir+'\Mozilla Thunderbird\thunderbird.exe') then begin
     AProcess.CommandLine:=pdir+'\Mozilla Thunderbird\thunderbird.exe /mailurl:mailto:B.Gross@pimicos.com?subject='+s+pf;
   end
   else if FileExists(pdir+'\Outlook Express\msimn.exe') then begin
    AProcess.CommandLine:=pdir+'\Outlook Express\msimn.exe /mailurl:mailto:B.Gross@pimicos.com?subject='+s+pf;
   end
   else if FileExists(pdir+'\Office\Office\Outlook.exe') then begin
     AProcess.CommandLine:=pdir+'\Office\Office\Outlook.exe /mailurl:mailto:B.Gross@pimicos.com?subject='+s+pf;
   end
   else if FileExists(pdir+'\Microsoft Office\Office14\Outlook.exe') then begin
    AProcess.CommandLine:=pdir+'\Microsoft Office\Office14\outlook.exe /c ipm.note /m B.Gross@pimicos.com&subject='+s+pf;
   end;
  {$ENDIF}
  if AProcess.CommandLine<>'' then begin
   AProcess.Execute;
  end
  else ShowMessage('Error creating mail!');
  AProcess.Free;
end;

initialization
  {$I u_about.lrs}
end.

