{***************************************************************************
 *   Copyright (C) 2006-2013 by Bernhard Gross                             *
 *   b.gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_cfg_dynamic_parm;

{$mode objfpc}

interface

uses
   Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, Buttons, Spin, ExtCtrls,u_exchange,u_talktosmc, LMessages,LCLType;

type

  { Tf_cfg_dynamic_parm }

  Tf_cfg_dynamic_parm = class(TForm)
    btn_close: TBitBtn;
    btn_set: TBitBtn;
    ed_acc: TLabeledEdit;
    ed_calin: TLabeledEdit;
    ed_calout: TLabeledEdit;
    ed_rmin: TLabeledEdit;
    ed_rmout: TLabeledEdit;
    ed_stpdcl: TLabeledEdit;
    ed_vel: TLabeledEdit;
    gb_cal: TGroupBox;
    gb_rm: TGroupBox;
    lbx_set: TListBox;
    spe_axis: TSpinEdit;
    procedure btn_setClick(Sender: TObject);
    procedure FormShortCut(var Msg: TLMKey; var Handled: Boolean);
    procedure FormShow(Sender: TObject);
    procedure spe_axisChange(Sender: TObject);
    procedure spe_axisKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

  private
    function readvalue(cmd:string):string;
    procedure readvar;
    function vc2_err: boolean;
    { private declarations }
  public
    { public declarations }
  end;

var
  f_cfg_dynamic_parm: Tf_cfg_dynamic_parm;

implementation

{ Tf_cfg_dynamic_parm }

procedure Tf_cfg_dynamic_parm.FormShow(Sender: TObject);
begin
 if ctrl_ident='hydra' then delete_error('ge',0);
 lbx_set.Clear;
 ed_vel.Clear; ed_vel.Color:=clwhite;
 ed_acc.Clear; ed_acc.Color:=clwhite;
 ed_stpdcl.Clear; ed_stpdcl.Color:=clwhite;
 ed_calin.Clear; ed_calin.Color:=clwhite;
 ed_calout.Clear; ed_calout.Color:=clwhite;
 ed_rmin.Clear; ed_rmin.Color:=clwhite;
 ed_rmout.Clear; ed_rmout.Color:=clwhite;
 spe_axis.MinValue:= LowAxisNo;
 spe_axis.MaxValue:=HighAxisNo;
 spe_axis.Enabled:=smc_count>1;
 spe_axis.Value:=CAxis;
 btn_set.Enabled:=smc_avail[spe_axis.Value];
 if smc_avail[spe_axis.Value] then readvar;
end;

procedure Tf_cfg_dynamic_parm.spe_axisChange(Sender: TObject);
begin
 btn_set.Enabled:=smc_avail[spe_axis.Value];
 ed_vel.Enabled:=smc_avail[spe_axis.Value];
 ed_acc.Enabled:=smc_avail[spe_axis.Value];
 ed_stpdcl.Enabled:=smc_avail[spe_axis.Value];
 ed_calin.Enabled:=smc_avail[spe_axis.Value];
 ed_calout.Enabled:=smc_avail[spe_axis.Value];
 ed_rmin.Enabled:=smc_avail[spe_axis.Value];
 ed_rmout.Enabled:=smc_avail[spe_axis.Value];
 lbx_set.Clear;
 if smc_avail[spe_axis.Value] then readvar
 else begin
  ed_vel.Clear; ed_vel.Color:=clwhite;
  ed_acc.Clear; ed_acc.Color:=clwhite;
  ed_stpdcl.Clear; ed_stpdcl.Color:=clwhite;
  ed_calin.Clear; ed_calin.Color:=clwhite;
  ed_calout.Clear; ed_calout.Color:=clwhite;
  ed_rmin.Clear; ed_rmin.Color:=clwhite;
  ed_rmout.Clear; ed_rmout.Color:=clwhite;
 end;
end;

procedure Tf_cfg_dynamic_parm.spe_axisKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var nw,err:integer;
begin
 if ord(key)=VK_RETURN then begin
  val (spe_axis.Caption,nw,err);
  if (err=0) and (nw in [1..Last_Ctrl]) then begin
   spe_axis.Value:=nw;
   spe_axisChange(nil);
  end;
 end;
end;

function Tf_cfg_dynamic_parm.readvalue(cmd: string): string;
var rslt,s:string;
begin
 rslt:='';
 s:= trim (smc_cmd_v2('',cmd,spe_axis.Value,1)); sleep (30);
 if s<>'' then begin
  rslt:=s;
 end
 else begin
  smc_cmd_v2('','gne',spe_axis.Value,1); sleep (30);
  smc_cmd_v2('','nclear',spe_axis.Value,0); sleep (30);
 end;
 readvalue:=rslt;
end;

procedure Tf_cfg_dynamic_parm.readvar;
var s,t:string;
begin
 vc2_err;
 lbx_set.Clear;
 ed_vel.Clear; ed_vel.Color:=clwhite;
 ed_acc.Clear; ed_acc.Color:=clwhite;
 ed_stpdcl.Clear; ed_stpdcl.Color:=clwhite;
 ed_calin.Clear; ed_calin.Color:=clwhite;
 ed_calout.Clear; ed_calout.Color:=clwhite;
 ed_rmin.Clear; ed_rmin.Color:=clwhite;
 ed_rmout.Clear; ed_rmout.Color:=clwhite;
 ed_vel.EditLabel.Caption:= 'Velocity '+actunit[spe_axis.Value]+'/s';
 ed_acc.EditLabel.Caption:= 'Acceleration '+actunit[spe_axis.Value]+'/s²';
 ed_stpdcl.EditLabel.Caption:= 'Stopdecel '+actunit[spe_axis.Value]+'/s²';
 ed_calin.EditLabel.Caption:= 'Velocity-In '+actunit[spe_axis.Value]+'/s';
 ed_calout.EditLabel.Caption:= 'Velocity-Out '+actunit[spe_axis.Value]+'/s';
 ed_rmin.EditLabel.Caption:= 'Velocity-In '+actunit[spe_axis.Value]+'/s';
 ed_rmout.EditLabel.Caption:= 'Velocity-Out '+actunit[spe_axis.Value]+'/s';
 ed_vel.Text:= readvalue('gnv');
 ed_vel.Enabled:=ed_vel.Text<>'';
 ed_acc.Text:= readvalue('gna');
 ed_acc.Enabled:=ed_acc.Text<>'';
 if ctrl_ident<>'hydra' then ed_stpdcl.Text:= readvalue('getnstopdecel') else ed_stpdcl.Text:= readvalue('gsd');
 ed_stpdcl.Enabled:=ed_stpdcl.Text<>'';
 t:=readvalue('getncalvel');
 s:=copy (t,1,pos(' ',t)-1);
 delete(t,1,pos(' ',t));
 ed_calin.Text:=s;
 ed_calin.Enabled:=s<>'';
 ed_calout.Text:=t;
 ed_calout.Enabled:=t<>'';
 t:=readvalue('getnrmvel');
 s:=copy (t,1,pos(' ',t)-1);
 delete(t,1,pos(' ',t));
 ed_rmin.Text:=s;
 ed_rmin.Enabled:=s<>'';
 ed_rmout.Text:=t;
 ed_rmout.Enabled:=s<>'';
end;

function Tf_cfg_dynamic_parm.vc2_err: boolean;
var s,t:string;
begin
 s:=trim (smc_cmd_v2('','gne',spe_axis.Value,1)); sleep (30);
 t:=trim (smc_cmd_v2('','ngsp',spe_axis.Value,1)); sleep (30);
 if (s='0') and (t='0') then vc2_err:=false else begin
  if t<>'0' then begin
   smc_cmd_v2('','nclear',spe_axis.Value,0); sleep (30);
  end;
  vc2_err:=true;
 end;
end;

procedure Tf_cfg_dynamic_parm.btn_setClick(Sender: TObject);
var res,t:string;
begin
 btn_close.Enabled:=false;
 vc2_err;
 lbx_set.Clear;
 if ed_vel.Enabled then begin
  ed_vel.Text:=StringReplace(ed_vel.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_vel.Text+' '+IntToStr(spe_axis.Value)+' snv');
  smc_cmd_v2(ed_vel.Text+'','snv',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_vel.Color:=clred else ed_vel.Color:=clWhite;
  Application.ProcessMessages;
  ed_vel.Text:= readvalue('gnv');
 end;
 if ed_acc.Enabled then begin
  ed_acc.Text:=StringReplace(ed_acc.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_acc.Text+' '+IntToStr(spe_axis.Value)+' sna');
  smc_cmd_v2(ed_acc.Text+'','sna',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_acc.Color:=clred else ed_acc.Color:=clwhite;
  Application.ProcessMessages;
  ed_acc.Text:= readvalue('gna');
 end;
 if ed_stpdcl.Enabled then begin
  ed_stpdcl.Text:=StringReplace(ed_stpdcl.Text,',','.',[rfReplaceAll]);
  if ctrl_ident<>'hydra' then lbx_set.Items.Add(ed_stpdcl.Text+' '+IntToStr(spe_axis.Value)+' setnstopdecel')
  else lbx_set.Items.Add(ed_stpdcl.Text+' '+IntToStr(spe_axis.Value)+' ssd');
  if ctrl_ident<>'hydra' then smc_cmd_v2(ed_stpdcl.Text+'','setnstopdecel',spe_axis.Value,0)
  else smc_cmd_v2(ed_stpdcl.Text+'','ssd',spe_axis.Value,0);
  sleep (30);
  if vc2_err then ed_stpdcl.Color:=clred else ed_stpdcl.Color:=clwhite;
  Application.ProcessMessages;
  if ctrl_ident<>'hydra' then ed_stpdcl.Text:= readvalue('getnstopdecel') else ed_stpdcl.Text:= readvalue('gsd');
 end;
 if ed_calin.Enabled then begin
  ed_calin.Text:=StringReplace(ed_calin.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_calin.Text+' 1 '+IntToStr(spe_axis.Value)+' setncalvel');
  smc_cmd_v2(ed_calin.Text+' 1','setncalvel',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_calin.Color:=clred else ed_calin.Color:=clwhite;
  Application.ProcessMessages;
  t:=readvalue('getncalvel');
  res:=copy (t,1,pos(' ',t)-1);
  ed_calin.Text:=res;
 end;
 if ed_calout.Enabled then begin
  ed_calout.Text:=StringReplace(ed_calout.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_calout.Text+' 2 '+IntToStr(spe_axis.Value)+' setncalvel');
  smc_cmd_v2(ed_calout.Text+' 2','setncalvel',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_calout.Color:=clred else ed_calout.Color:=clwhite;
  Application.ProcessMessages;
  res:=readvalue('getncalvel');
  delete (res,1,pos(' ',res));
  ed_calout.Text:=res;
 end;
 if ed_rmin.Enabled then begin
  ed_rmin.Text:=StringReplace(ed_rmin.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_rmin.Text+' 1 '+IntToStr(spe_axis.Value)+' setnrmvel');
  smc_cmd_v2(ed_rmin.Text+' 1','setnrmvel',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_rmin.Color:=clred else ed_rmin.Color:=clwhite;
  Application.ProcessMessages;
  t:=readvalue('getnrmvel');
  res:=copy (t,1,pos(' ',t)-1);
  ed_rmin.Text:=res;
 end;
 if ed_rmout.Enabled then begin
  ed_rmout.Text:=StringReplace(ed_rmout.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_rmout.Text+' 2 '+IntToStr(spe_axis.Value)+' setnrmvel');
  smc_cmd_v2(ed_rmout.Text+' 2','setnrmvel',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_rmout.Color:=clred else ed_rmout.Color:=clwhite;
  Application.ProcessMessages;
  res:=readvalue('getnrmvel');
  delete (res,1,pos(' ',res));
  ed_rmout.Text:=res;
 end;
 btn_close.Enabled:=true;
end;

procedure Tf_cfg_dynamic_parm.FormShortCut(var Msg: TLMKey; var Handled: Boolean);
begin
 if msg.CharCode=27 then smc_cmd_v2(#3,'',0,0);
end;

initialization
  {$I u_cfg_dynamic_parm.lrs}

end.

