{***************************************************************************
 *   Copyright (C) 2006-2013 by Bernhard Gross                             *
 *   b.gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_cfg_stages;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Spin, LMessages,u_exchange,u_talktosmc, LCLType;

type

  { Tf_stages }

  Tf_stages = class(TForm)
    btn_close: TBitBtn;
    btn_set: TBitBtn;
    cbx_axis: TComboBox;
    cbx_e1: TComboBox;
    cbx_e2: TComboBox;
    ed_cald: TLabeledEdit;
    ed_pitch: TLabeledEdit;
    gb_sw1: TGroupBox;
    gbx_initlimit: TGroupBox;
    ed_llmt: TLabeledEdit;
    ed_ulmt: TLabeledEdit;
    lb_e1: TLabel;
    lb_e2: TLabel;
    lbx_set: TListBox;
    lb_unit1: TLabel;
    lb_unit2: TLabel;
    rg_mdir: TRadioGroup;
    spe_axis: TSpinEdit;
    procedure btn_setClick(Sender: TObject);
    procedure FormShortCut(var Msg: TLMKey; var Handled: Boolean);
    procedure FormShow(Sender: TObject);
    procedure spe_axisChange(Sender: TObject);
    procedure spe_axisKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState
      );
  private
    dec_str:string;
    function readvalue(cmd: string): string;
    procedure readvar;
    function vc2_err: boolean;
    { private declarations }
  public
    { public declarations }
  end; 

var
  f_stages: Tf_stages;

implementation

{ Tf_stages }

function Tf_stages.readvalue(cmd:string):string;
var rslt,s:string;
begin
 rslt:='';
 s:= trim (smc_cmd_v2('',cmd,spe_axis.Value,1)); sleep (30);
 if s<>'' then begin
  rslt:=s;
 end
 else begin
  smc_cmd_v2('','gne',spe_axis.Value,1); sleep (30);
  smc_cmd_v2('','nclear',spe_axis.Value,0); sleep (30);
 end;
 readvalue:=rslt;
end;



function Tf_stages.vc2_err:boolean;
var s,t:string;
begin
 s:=trim (smc_cmd_v2('','gne',spe_axis.Value,1)); sleep (30);
 t:=trim (smc_cmd_v2('','ngsp',spe_axis.Value,1)); sleep (30);
 if (s='0') and (t='0') then vc2_err:=false else begin
  if t<>'0' then begin
   smc_cmd_v2('','nclear',spe_axis.Value,0); sleep (30);
  end;
  vc2_err:=true;
 end;
end;

procedure Tf_stages.readvar;
var s,t:string;
    k,err:integer;
begin
 vc2_err;
 lbx_set.Clear;
 ed_pitch.Clear; ed_pitch.Color:=clwhite;
 ed_cald.Clear; ed_cald.Color:=clwhite;
 ed_llmt.Color:=clWhite; ed_ulmt.Color:=clWhite;
 s:= readvalue('getaxis');
 cbx_axis.Enabled:=s<>'';
 val (s,k,err);
 if err<>0 then k:=1;
 cbx_axis.ItemIndex:=k;
 s:= readvalue('getpitch');
 ed_pitch.Text:=s;
 s:=readvalue('getsw');
 t:=s[1]; val (t,k,err);
 if err<>0 then k:=1;
 cbx_e1.ItemIndex:=k;
 t:=s[3]; val (t,k,err);
 if err<>0 then k:=1;
 cbx_e2.ItemIndex:=k;
 s:= readvalue('getncalswdist');
 ed_cald.Enabled:=s<>'';
 ed_cald.Text:=s;
 s:= readvalue('getmotiondir');
 rg_mdir.Enabled:=s<>'';
 if s<>'' then begin
  t:=s[1]; val (t,k,err);
  if err<>0 then k:=1;
  rg_mdir.ItemIndex:=k;
 end;
 s:= readvalue('getinilimit');
 ed_llmt.Enabled:=s<>'';
 ed_ulmt.Enabled:=s<>'';
 if s<>'' then begin
  t:= copy (s,1,pos(' ',s)-1);
  ed_llmt.Text:=t;
  delete (s,1,pos (' ',s));
  ed_ulmt.text:=s;
 end;
end;

procedure Tf_stages.FormShortCut(var Msg: TLMKey; var Handled: Boolean);
begin
 if msg.CharCode=27 then smc_cmd_v2(#3,'',0,0); 
end;

procedure Tf_stages.btn_setClick(Sender: TObject);
var s,t,res:string;
    nv:double;
    k,err:integer;
begin
 btn_close.Enabled:=false;
 vc2_err;
 lbx_set.Clear;
 ed_pitch.Color:=clwhite;
 ed_cald.Color:=clwhite;
 ed_llmt.Color:=clWhite;
 ed_ulmt.Color:=clWhite;
 lbx_set.Clear;
 s:=''; t:='';
 // setaxis
 if cbx_axis.Enabled then begin
  lbx_set.Items.Add(IntToStr(cbx_axis.ItemIndex)+' '+IntToStr(spe_axis.Value)+' setaxis');
  smc_cmd_v2(IntToStr(cbx_axis.ItemIndex)+'','setaxis',spe_axis.Value,0); sleep (30);
  res:= readvalue('getaxis');
  val (res,k,err);
  if err<>0 then k:=1;
  cbx_axis.ItemIndex:=k;
 end;
 // setpitch
 nv:=1;
 ed_pitch.Text:=StringReplace(ed_pitch.Text,',','.',[rfReplaceAll]);
 val (ed_pitch.Text,nv,err);
 s:= FormatFloat(dec_str,nv);
 lbx_set.Items.Add(s+' '+IntToStr(spe_axis.Value)+' setpitch');
 smc_cmd_v2(s+'','setpitch',spe_axis.Value,0); sleep (30);
 res:= readvalue('getpitch');
 ed_pitch.Text:=res;
 if vc2_err then ed_pitch.Color:=clred else ed_pitch.Color:=clWhite;
 // setsw
 lbx_set.Items.Add(IntToStr(cbx_e1.ItemIndex)+' 0 '+IntToStr(spe_axis.Value)+' setsw');
 smc_cmd_v2(IntToStr(cbx_e1.ItemIndex)+' 0','setsw',spe_axis.Value,0); sleep (30);
 lbx_set.Items.Add(IntToStr(cbx_e2.ItemIndex)+' 1 '+IntToStr(spe_axis.Value)+' setsw');
 smc_cmd_v2(IntToStr(cbx_e2.ItemIndex)+' 1','setsw',spe_axis.Value,0); sleep (30);
 res:=readvalue('getsw');
 t:=res[1]; val (t,k,err);
 if err<>0 then k:=1;
 cbx_e1.ItemIndex:=k;
 t:=res[3]; val (t,k,err);
 if err<>0 then k:=1;
 cbx_e2.ItemIndex:=k;
 // setcalswdist
 if ed_cald.Enabled then begin
  ed_cald.Text:=StringReplace(ed_cald.Text,',','.',[rfReplaceAll]);
  lbx_set.Items.Add(ed_cald.Text+' '+IntToStr(spe_axis.Value)+' setncalswdist');
  smc_cmd_v2(ed_cald.Text+'','setncalswdist',spe_axis.Value,0); sleep (30);
  if vc2_err then ed_cald.Color:=clred else ed_cald.Color:=clWhite;
  res:= readvalue('getncalswdist');
  ed_cald.Text:=res;
 end;
 // setmotiondir
 if rg_mdir.Enabled then begin
  lbx_set.Items.Add(IntToStr(rg_mdir.ItemIndex)+' '+IntToStr(spe_axis.Value)+' setmotiondir');
  smc_cmd_v2(IntToStr(rg_mdir.ItemIndex)+'','setmotiondir',spe_axis.Value,0); sleep (30);
  res:= readvalue('getmotiondir');
  t:=res[1]; val (t,k,err);
  if err<>0 then k:=1;
  rg_mdir.ItemIndex:=k;
 end;
 if ed_llmt.Enabled and ed_ulmt.Enabled then begin
  ed_llmt.Text:=StringReplace(ed_llmt.Text,',','.',[rfReplaceAll]);
  ed_ulmt.Text:=StringReplace(ed_ulmt.Text,',','.',[rfReplaceAll]);
  nv:=1;
  val (ed_llmt.Text,nv,err);
  s:= FormatFloat(dec_str,nv);
  nv:=1;
  val (ed_ulmt.Text,nv,err);
  t:= FormatFloat(dec_str,nv);
  lbx_set.Items.Add(s+' '+t+' '+IntToStr(spe_axis.Value)+' setinilimit');
  smc_cmd_v2(s+' '+t+'','setinilimit',spe_axis.Value,0); sleep (30);
  if vc2_err then begin
   ed_llmt.Color:=clred; ed_ulmt.Color:=clred;
  end
  else begin
   ed_llmt.Color:=clWhite; ed_ulmt.Color:=clWhite;
  end;
 end;
 btn_close.Enabled:=true;
end;

procedure Tf_stages.FormShow(Sender: TObject);
begin
 lbx_set.Clear;
 if ctrl_ident='hydra' then delete_error('ge',0);
 if (ctrl_ident='pollux') or (ctrl_ident='pollux-nt')then dec_str:='0.0000' else dec_str:='0.000000';
 ed_pitch.Clear; ed_pitch.Color:=clwhite;
 ed_cald.Clear; ed_cald.Color:=clwhite;
 ed_llmt.Color:=clWhite; ed_ulmt.Color:=clWhite;
 spe_axis.MinValue:= LowAxisNo;
 spe_axis.MaxValue:=HighAxisNo;
 spe_axis.Enabled:=smc_count>1;
 spe_axis.Value:=CAxis;
 lb_unit1.Caption:=actunit[spe_axis.Value];
 lb_unit2.Caption:=actunit[spe_axis.Value];
 cbx_axis.Items.LoadFromFile(ProgramDir+DirSep+'controller'+DirSep+ctrl_dir+DirSep+'setaxis.txt');
 btn_set.Enabled:=smc_avail[spe_axis.Value];
 if smc_avail[spe_axis.Value] then readvar;
end;

procedure Tf_stages.spe_axisChange(Sender: TObject);
begin
 btn_set.Enabled:=smc_avail[spe_axis.Value];
 cbx_axis.Enabled:=smc_avail[spe_axis.Value];
 cbx_e1.Enabled:=smc_avail[spe_axis.Value];
 cbx_e2.Enabled:=smc_avail[spe_axis.Value];
 rg_mdir.Enabled:=smc_avail[spe_axis.Value];
 ed_cald.Enabled:=smc_avail[spe_axis.Value];
 ed_pitch.Enabled:=smc_avail[spe_axis.Value];
 gbx_initlimit.Enabled:=smc_avail[spe_axis.Value];
 lbx_set.Clear;
 if smc_avail[spe_axis.Value] then readvar
 else begin
  ed_pitch.Clear; ed_pitch.Color:=clwhite;
  ed_cald.Clear; ed_cald.Color:=clwhite;
  ed_llmt.Color:=clWhite; ed_ulmt.Color:=clWhite;
 end;
end;

procedure Tf_stages.spe_axisKeyDown(Sender: TObject; var Key: Word;  Shift: TShiftState);
var nw,err:integer;
begin
 if ord(key)=VK_RETURN then begin
  val (spe_axis.Caption,nw,err);
  if (err=0) and (nw in [1..Last_Ctrl]) then begin
   spe_axis.Value:=nw;
   spe_axisChange(nil);
  end;
 end;
end;

initialization
  {$I u_cfg_stages.lrs}

end.

