{***************************************************************************
 *   Copyright (C) 2006-2013 by Bernhard Gross                             *
 *   b.gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_exchange;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,graphics,synaser;

var actdev,actbaud,actserialno:string;
    CAxis,smc_return,smc_cmdtout:integer;
    smc_cmd_active,abreak,axis_moves,bshowhint:boolean;
    newinc:double;   //Floatspinedit.value target
    decplc:integer;   //Floatspinedit.Decimalplaces target
    bok:boolean;
    acl_ctrl:array[1..16] of boolean; //closed-loop controller
    venusfile,editstr:string;
    dl,iii:integer;
    sc_fac:array[1..16] of double; //define the scale-factor for the axes
    display_pos:array[1..16] of string; //define the format the position display
    System_DecSep:char;
    DirSep:char;       //DirectorySeparator
    ProgramDir:string; //Program-Directory
    Lin_Unit:String; //Unit for linear-axes
    Rot_Unit:String; //Unit for rotation-axes
    actunit :array[1..16] of string;
    LowAxisNo,HighAxisNo:integer;
    cmdlst:TStringList;
    CtrlLst:TStringList; //List of controller and parameters
    newvc2:boolean;
    v2cmd:boolean; //Commando axis independing (Hydra)
    ctrl_dir:string;
    smc_getparm:string;  // getparameter setparameter
    smc_cmd:string;  // getparameter setparameter
    smc_blckcmd:boolean;
    ctrl_ident:string;
    tbl_loaded :boolean; //User Tableau loaded
    pollingtime:integer;
    spdmode:boolean;
    warn_diff_ctrl,diff_ctrl:boolean; //different controller/firmware
    Last_Ctrl,Cfg_last_Ctrl:Integer; //Defines for Pollux and Pegasus the last Controller
    failport:boolean; //Support for buggy ports
    tcpip_select:boolean; //Ethernet or RS-232
    tcpip_adr,tcpip_port:string; //Ethernet parameters
    VirtPollux:boolean; //Virtuell Pollux for demo
    tmr_exec:boolean; // Main Form Timer polling on/off
    ans_be_str:boolean; //Answer is a string
    serial_str:string; //SerialNo of Controller;
    axes_rbbt:Integer;
    spdwin_open:boolean;
    spdstps:array [1..16] of integer;
    spdcur:array [1..16] of integer;
    hwmv:shortint; //handwheel move
    showhwmd:boolean; //Show Handweel Mode
    hwcnt:byte;
    alv:byte; //Alive Display
    tmr_strt:boolean; //Debug Timer Start

const progvers:string='Version 0.28.00';
      smc_timeout:Integer=175;
      alive:array [1..4] of char=('-','\','|','/');

function Bool_To_Str (B:boolean):string;
function Get_Serial_PortNames: string;
function cmd_check(cmd: string): boolean;
function ctrl_parm:boolean;

implementation

function Bool_To_Str (B:boolean):string;
begin
 if b=true then Bool_To_Str:='TRUE' else Bool_To_Str:='FALSE';
end;

function cmd_check (cmd: string): boolean;
var res:boolean;
    s,sCmd,sRet,sTmo,sPara,dmy,vc1:string;
    i,err:Integer;
begin
 res:=true; s:=trim (cmd); dmy:=''; i:=0; smc_cmd:='';
 smc_getparm:=''; smc_return:=0; smc_cmdtout:=0;
 if s<>'' then begin
  repeat
   delete (s,1,pos(' ',s));
   s:=trim (s);
  until (pos(' ',s)=0);
 end;
 while (i<CmdLst.Count) and (dmy='') do begin
  if pos (LowerCase(s)+';',LowerCase (CmdLst[i]))=1 then dmy:=CmdLst[i];
  if dmy='' then inc (i);
 end;
 if dmy<>'' then begin
  sCmd:=copy (dmy,1,pos(';',dmy)-1);
  smc_cmd:=sCmd;
  delete (dmy,1,pos(';',dmy));
  if sCmd='' then res:=false;
  sRet:=copy (dmy,1,pos(';',dmy)-1);
  delete (dmy,1,pos(';',dmy));
  val (sRet,smc_return,err);
  if err<>0 then res:=false;
  sTmo:=copy (dmy,1,pos(';',dmy)-1);
  delete (dmy,1,pos(';',dmy));
  val (sTmo,smc_cmdtout,err);
  if err<>0 then res:=false;
  sPara:=copy (dmy,1,pos(';',dmy)-1);
  delete (dmy,1,pos(';',dmy));
  if sPara<>'' then smc_getparm:=sPara else res:=false;
  vc1:=LowerCase(copy (dmy,1,pos(';',dmy)-1));
  delete (dmy,1,pos(';',dmy));
  if vc1='true' then v2cmd:=true else v2cmd:=false;
 end
 else res:=false;
 cmd_check:=res;
end;

function Get_Serial_PortNames: string;
var sr : TSearchRec;
    portstr:string;
begin
  portstr:='';
 {$IFDEF LINUX}
   if FindFirst('/dev/ttyS*',faAnyFile,sr) = 0 then begin
    repeat
     portstr:=portstr+'/dev/'+sr.Name+',';
    until FindNext(sr) <> 0;
    end;
   FindClose(sr);
   if FindFirst('/dev/ttyUSB*',faAnyFile,sr) = 0 then begin
    repeat
     portstr:=portstr+'/dev/'+sr.Name+',';
    until FindNext(sr) <> 0;
    end;
   FindClose(sr);
   delete (portstr,length (portstr),1);
 {$ENDIF}
 {$IFDEF WINDOWS}
  portstr:=GetSerialPortNames;
 {$ENDIF}
 Get_Serial_PortNames:=portstr;
end;


function ctrl_parm:boolean;
var ctrl_file:TextFile;
    err: integer;
    s:string;
begin
 err:=0;
 Last_Ctrl:=16;
 CtrlLst:=TStringList.Create;
 CtrlLst.Clear;
 AssignFile(ctrl_file,ProgramDir+DirSep+'controller'+DirSep+'controller.txt');
 {$I-}
 reset (ctrl_file);
 inc (err,IOResult);
 ReadLn (ctrl_file,s); //dummy
 inc (err,IOResult);
 while (not eof (ctrl_file)) and (err=0) do begin
  ReadLn (ctrl_file,s);
  inc (err,IOResult);
  CtrlLst.add (s);
 end;
 CloseFile(ctrl_file);
 inc (err,IOResult);
 {$I+}
 ctrl_parm:=err=0;
end;


begin
 alv:=1;
 tmr_exec:=true;
 failport:=false;
 tbl_loaded:=false;
 VirtPollux:=false;
 newvc2:=true;
 serial_str:='';
 axes_rbbt:=0;
 spdwin_open:=false;
 showhwmd:=false;
 for iii:= 1 to 16 do begin
  spdstps[iii]:=13;
  spdcur[iii]:=-1;
 end;
end.

