{***************************************************************************
 *   Copyright (C) 2006-2013 by Bernhard Gross                             *
 *   b.gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_export_cfg;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls,Buttons
  ,Spin, ExtCtrls,u_talktosmc,u_exchange, u_dbg, LCLType;

type

  { TF_export }

  TF_export = class(TForm)
    btn_stop: TBitBtn;
    btn_read: TBitBtn;
    btn_close: TBitBtn;
    btn_save: TBitBtn;
    lb_cnt: TLabel;
    mm_rslt: TMemo;
    SaveDialog1: TSaveDialog;
    spe_axis: TSpinEdit;
    procedure btn_readClick(Sender: TObject);
    procedure btn_saveClick(Sender: TObject);
    procedure btn_stopClick(Sender: TObject);
    procedure FormClose(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure spe_axisChange(Sender: TObject);
    procedure spe_axisKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
   ReadCmdLst:TStringList;
   cnt:Integer;
   abbrt:boolean;
   procedure create_cmd_list;
   procedure read_cfg;
    { private declarations }
  public
    { public declarations }
  end; 

var
  F_export: TF_export;
  filetosave:boolean;
     
implementation

{ TF_export }



procedure TF_export.create_cmd_list;
var s,t,u,axstr:string;
    i,j,a,b,err:integer;
begin
 j:=1;
 for i:= 0 to cmdlst.Count-1 do begin
  s:=copy (cmdlst[i],1,pos(';',cmdlst[i])-1);
  cmd_check(s);
  if v2cmd then axstr:=' # ' else axstr:=' ';
  if smc_return>0 then begin
   if smc_getparm='0-0' then begin
    if s<>'getaxisno' then ReadCmdLst.add (axstr+s);
   end
   else if pos ('-',smc_getparm)>0 then begin
    t:=copy (smc_getparm,1,Pos('-',smc_getparm)-1);
    val (t,a,err); if err<>0 then a:=0;
    t:=smc_getparm; delete (t,1,Pos('-',smc_getparm));
    val (t,b,err); if err<>0 then b:=0;
    for j:= a to b do ReadCmdLst.add (IntToStr(j)+axstr+s);
   end
   else if pos (',',smc_getparm)>0 then begin
    t:=smc_getparm;
    if t[Length(t)] <>',' then t:=t+',';
    repeat
     u:=copy (t,1,pos(',',t)-1);
     delete (t,1,pos(',',t));
     val (u,j,err);
     if err=0 then ReadCmdLst.add (IntToStr(j)+axstr+s);
    until trim (t)='';
   end;
  end;
 end;
end;


procedure TF_export.FormShow(Sender: TObject);
var i:integer;
begin
 if ctrl_ident='hydra' then delete_error('ge',0);
 ReadCmdLst:=TStringList.Create;
 ReadCmdLst.Clear;
 btn_stop.Enabled:=false;
 spe_axis.Enabled:=smc_count>1;
 create_cmd_list;
 lb_cnt.Caption:='Commands = '+ IntToStr(ReadCmdLst.Count);
 btn_save.Enabled:=false;
 filetosave:=False;
 spe_axis.MaxValue:=0;
 for i:= 1 to Last_Ctrl do if smc_avail[i] then spe_axis.MaxValue:=i;
 spe_axis.Value:=CAxis;
 btn_read.Enabled:=smc_avail[spe_axis.Value];
end;

procedure TF_export.spe_axisChange(Sender: TObject);
begin
 btn_read.Enabled:=smc_avail[spe_axis.Value];
 mm_rslt.Clear;
 btn_save.Enabled:=false;
end;

procedure TF_export.spe_axisKeyDown(Sender: TObject; var Key: Word;  Shift: TShiftState);
var nw,err:integer;
begin
 if ord(key)=VK_RETURN then begin
  val (spe_axis.Caption,nw,err);
  if (err=0) and (nw in [1..Last_Ctrl]) then begin
   spe_axis.Value:=nw;
   spe_axisChange(nil);
  end;
 end;
end;

procedure TF_export.read_cfg;
var s,cmd:string;
begin
 abbrt:=false; cnt:=0;
 while (cnt<ReadCmdLst.Count) and (abbrt=false) do begin
  cmd:=StringReplace(ReadCmdLst[cnt],'#',spe_axis.Caption,[rfReplaceAll]);
  mm_rslt.Lines.Add(cmd);
  if cmd<>'getaxisno' then begin
  s:=trim(smc_cmd_v2(cmd,'',0,1));
  sleep (pollingtime);
  if dbg_wrt then begin
    sleep (pollingtime);
    if ctrl_ident<>'hydra' then smc_cmd_v2('','gne',CAxis,1) else smc_cmd_v2('ge','',0,1);
    sleep (pollingtime);
    if ctrl_ident<>'hydra' then smc_cmd_v2('','ngsp',CAxis,1) else smc_cmd_v2('gsp','',0,1);
   end;
  end
  else s:=IntToStr(spe_axis.Value);
  mm_rslt.Lines.Add('   '+s);
  Application.ProcessMessages;
  inc (cnt);
 end;
 sleep (pollingtime);
 smc_cmd_v2('','gne',CAxis,1);
 sleep (pollingtime);
 smc_cmd_v2('','nclear',CAxis,0);
 btn_save.Enabled:=true;
 btn_read.Enabled:=true;
 btn_close.Enabled:=true;
 spe_axis.Enabled:=true;
 F_export.Cursor:=crDefault;
end;


procedure TF_export.btn_saveClick(Sender: TObject);
var fn:string;
    i:integer;
begin
 i:=0; fn:='';
 while (i<mm_rslt.Lines.Count) and (pos('serial',mm_rslt.Lines[i])=0) do inc (i);
 if (ctrl_ident='taurus') or (ctrl_ident='pegasus') then fn:=serial_str;
 fn:=Trim (mm_rslt.Lines[i+1]);
 while length(fn) <8 do fn:='0'+fn;
 if (ctrl_ident='hydra') or(ctrl_ident='taurus') or (ctrl_ident='pegasus') then fn:=fn+'-AxisNo-'+IntToStr(CAxis);
 SaveDialog1.InitialDir:=ProgramDir;
 SaveDialog1.Filter:='Text-Files (*.txt)|*.txt|All Files (*.*)|*.*';
 SaveDialog1.DefaultExt:='.txt';
 SaveDialog1.FileName:='cfg_'+fn+'.txt';
 if (SaveDialog1.Execute) and (SaveDialog1.FileName<>SaveDialog1.InitialDir) then begin
  mm_rslt.Lines.SaveToFile(SaveDialog1.FileName);
 end;
end;

procedure TF_export.btn_stopClick(Sender: TObject);
begin
  abbrt:=true;
end;

procedure TF_export.FormClose(Sender: TObject);
begin
 ReadCmdLst.free;
 mm_rslt.Clear;
end;

procedure TF_export.btn_readClick(Sender: TObject);
var s:string;
begin
 s:='';
 F_export.Cursor:=crHourGlass;
 btn_close.Enabled:=false;
 btn_read.Enabled:=false;
 spe_axis.Enabled:=false;
 btn_save.Enabled:=false;
 cnt:=0;
 mm_rslt.Clear;
 smc_cmd_v2('','gne',CAxis,1);
 sleep (pollingtime);
 s:=trim(smc_cmd_v2('','ngsp',CAxis,1));
 sleep (pollingtime);
 if s<>'0' then smc_cmd_v2('','nclear',CAxis,0);
 if ctrl_ident<>'hydra'then begin
  mm_rslt.Lines.add ('Pollux-Term internal scalefactor:');
  mm_rslt.Lines.add ('   '+FormatFloat('0.0000000000',sc_fac[1]));
 end;
 btn_stop.Enabled:=true;
 read_cfg;
 btn_stop.Enabled:=false;
end;

initialization
  {$I u_export_cfg.lrs}
end.

