{***************************************************************************
 *   Copyright (C) 2006-2013 by Bernhard Gross                             *
 *   b.gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_scalefactors;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Buttons, StdCtrls,u_exchange, Spin,math,LCLType,u_talktosmc;

type

  { Tf_scale }

  Tf_scale = class(TForm)
    btn_cancel: TBitBtn;
    btn_ok: TBitBtn;
    btn_clear: TBitBtn;
    ed_scalefac: TLabeledEdit;
    Label1: TLabel;
    lb_c_vel: TLabel;
    lb_s_vel: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lb_c_pth: TLabel;
    lb_s_pth: TLabel;
    Label6: TLabel;
    lb_c_pos: TLabel;
    lb_s_pos: TLabel;
    Label9: TLabel;
    mm_scale: TMemo;
    spe_axis: TSpinEdit;
    procedure btn_okClick(Sender: TObject);
    procedure btn_clearClick(Sender: TObject);
    procedure ed_scalefacChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ed_scalefacExit(Sender: TObject);
    procedure spe_axisChange(Sender: TObject);
    procedure spe_axisKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState
      );
  private
   nsc_fac: array[1..16] of double;
   procedure ScaledValues;
    { private declarations }
  public
    { public declarations }
  end; 

var
  f_scale: Tf_scale;

implementation

{ Tf_scale }
procedure Tf_scale.ScaledValues;
var rv:double;
    s:string;
    err:Integer;
begin
 lb_c_pos.Caption:=''; lb_c_pth.Caption:='';lb_c_vel.Caption:='';
 lb_s_pos.Caption:=''; lb_s_pth.Caption:='';lb_s_vel.Caption:='';
 if smc_avail[spe_axis.Value] then begin
  s:=trim (smc_cmd_v2('','getpitch',spe_axis.Value,1));
  val (s,rv,err);
  if err=0 then begin
   lb_c_pth.Caption:=FormatFloat('0.0000000000',rv)+'mm';
   lb_s_pth.Caption:=FormatFloat('0.0000000000',rv*nsc_fac[spe_axis.Value])+actunit[spe_axis.Value];
  end;
  rv:=1;
  if err=0 then begin
   lb_c_pos.Caption:=FormatFloat('0.0000000000',rv/nsc_fac[spe_axis.Value])+'mm';
   lb_s_pos.Caption:=FormatFloat('0.0000000000',rv)+actunit[spe_axis.Value];
  end;
  s:=trim (smc_cmd_v2('','gnv',spe_axis.Value,1));
  val (s,rv,err);
   if err=0 then begin
   lb_c_vel.Caption:=FormatFloat('0.0000000000',rv)+'mm/s';
   lb_s_vel.Caption:=FormatFloat('0.0000000000',rv*nsc_fac[spe_axis.Value])+actunit[spe_axis.Value]+'/s';
  end;
 end;
end;

procedure Tf_scale.FormShow(Sender: TObject);
begin
  btn_clear.ShowHint:=bshowhint;
  nsc_fac:=sc_fac;
  ed_scalefac.Text:=FormatFloat('0.0000000000',sc_fac[spe_axis.Value]);
  ScaledValues;
end;

procedure Tf_scale.ed_scalefacExit(Sender: TObject);
var dmy:double;
    s:string;
    err:Integer;
begin
 dmy:=-1; err:=0;
 s:=ed_scalefac.Text;
 if pos(',',s)>0 then s[pos(',',s)]:='.';
 val (s,dmy,err);
 f_scale.ActiveControl:=btn_ok;
 if (err=0)and(dmy<>0) then nsc_fac[spe_axis.Value]:=dmy;
end;

procedure Tf_scale.spe_axisChange(Sender: TObject);
begin
 ed_scalefac.Text:=FormatFloat('0.0000000000',nsc_fac[spe_axis.Value]);
 ScaledValues;
end;

procedure Tf_scale.spe_axisKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var nw,err:integer;
begin
 if ord(key)=VK_RETURN then begin
  val (spe_axis.Caption,nw,err);
  if (err=0) and (nw in [1..Last_Ctrl]) then begin
   spe_axis.Value:=nw;
   spe_axisChange(nil);
  end;
 end;
end;

procedure Tf_scale.btn_clearClick(Sender: TObject);
begin
  nsc_fac[spe_axis.Value]:=1;
  ed_scalefac.Text:=FormatFloat('0.0000000000',nsc_fac[spe_axis.Value]);
  ScaledValues;
end;

procedure Tf_scale.ed_scalefacChange(Sender: TObject);
var err:integer;
    nv:double;
begin
  ed_scalefac.Text:= StringReplace(ed_scalefac.Text,',','.',[rfReplaceAll]);
  val (ed_scalefac.Text,nv,err);
  if (err=0) and (nv<>0) then begin
   nsc_fac[spe_axis.Value]:=nv;
   ScaledValues;
  end;
end;

procedure Tf_scale.btn_okClick(Sender: TObject);
var i:integer;
    tg:double;
begin
 sc_fac:=nsc_fac;
 for i:= 1 to 16 do begin
  display_pos[i]:='0000000000';
  tg:= int (log10(1000*sc_fac[i]))+2;
  insert ('.',display_pos[i],round(tg));
 end;
end;

initialization
  {$I u_scalefactors.lrs}

end.

