{***************************************************************************
 *   Copyright (C) 2006-2013 by Bernhard Gross                             *
 *   b.gross@pimicos.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************}

unit u_speedmode;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,u_exchange,
  Buttons, StdCtrls,LCLType, ComCtrls,u_talktosmc;

type

  { Tf_speedmode }

  Tf_speedmode = class(TForm)
    lb_vel: TLabel;
    btn_spd_neg: TSpeedButton;
    btn_spd_pos: TSpeedButton;
    btn_spd_dwn: TSpeedButton;
    btn_spd_up: TSpeedButton;
    procedure btn_spd_negMouseLeave(Sender: TObject);
    procedure btn_spd_posMouseLeave(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure btn_spd_negMouseDown(Sender: TObject; Button: TMouseButton;Shift: TShiftState; X, Y: Integer);
    procedure btn_spd_negMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_spd_posMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_spd_posMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_spd_dwnClick(Sender: TObject);
    procedure btn_spd_upClick(Sender: TObject);
  private
  spd_dir:Integer;
  spd_max:Double;
  cur_vel,cur_acc,lmt_p,lmt_n,trgt:String;
    { private declarations }
  public
  procedure show_vel;
  procedure get_lmt;
  procedure get_maxvel;
    { public declarations }
  end; 
const acc_vel:double=10; //acc = x times vel;
var
  f_speedmode: Tf_speedmode;

implementation

{ Tf_speedmode }

procedure Tf_speedmode.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
 if spd_dir=-1 then trgt:=lmt_n else trgt:=lmt_p;
 Case key of
  VK_LEFT: begin
   spd_dir:=-1; trgt:=lmt_n;
   show_vel;
   if ctrl_ident<>'hydra' then smc_cmd_v2(cur_vel,'speed',Caxis,0)
   else smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
  end;
  VK_RIGHT: begin
   spd_dir:=1; trgt:=lmt_p;
   show_vel;
   if ctrl_ident<>'hydra' then smc_cmd_v2(cur_vel,'speed',Caxis,0)
   else smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
  end;
  VK_UP: begin
   if spdcur[CAxis] <spdstps[CAxis] then inc (spdcur[CAxis]);
   show_vel;
   if spdmode and (ctrl_ident<>'hydra') then smc_cmd_v2(cur_vel,'speed',Caxis,0)
   else if spdmode then smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
  end;
  VK_DOWN: begin
   if spdcur[CAxis] >0 then dec (spdcur[CAxis]);
   show_vel;
   if spdmode and (ctrl_ident<>'hydra') then smc_cmd_v2(cur_vel,'speed',Caxis,0)
   else if spdmode then smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
  end;
  27: begin
   if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
  end;
 end;
end;

procedure Tf_speedmode.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
 if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
 spdwin_open:=false;
end;

procedure Tf_speedmode.FormDeactivate(Sender: TObject);
begin
 if spdmode then if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
end;

procedure Tf_speedmode.btn_spd_negMouseLeave(Sender: TObject);
begin
 if spdmode then if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
end;

procedure Tf_speedmode.btn_spd_posMouseLeave(Sender: TObject);
begin
 if spdmode then if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
end;

procedure Tf_speedmode.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 Case key of
   VK_LEFT: begin
    if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
  end;
   VK_RIGHT: begin
    if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
  end;
 end;
end;

procedure Tf_speedmode.get_maxvel;
var s:string;
    err:integer;
    res:double;
begin
 spd_max:=0;
 s:=Trim(smc_cmd_v2('','gnv',CAxis,1));
 val (s,res,err);
 if err=0 then spd_max:=res;
end;

procedure Tf_speedmode.FormShow(Sender: TObject);
begin
 if spdcur [CAxis]=-1 then spdcur[CAxis]:=1;
 spd_dir:=1;
 btn_spd_dwn.ShowHint:=bshowhint;
 btn_spd_up.ShowHint:=bshowhint;
 btn_spd_pos.ShowHint:=bshowhint;
 btn_spd_neg.ShowHint:=bshowhint;
 get_maxvel;
 show_vel;
 get_lmt;
end;

procedure Tf_speedmode.btn_spd_negMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 spd_dir:=-1; trgt:=lmt_n;
 show_vel;
 if ctrl_ident<>'hydra' then smc_cmd_v2(cur_vel,'speed',Caxis,0)
 else smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
end;

procedure Tf_speedmode.btn_spd_negMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
end;

procedure Tf_speedmode.btn_spd_posMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 spd_dir:=1; trgt:=lmt_p;
 show_vel;
 if ctrl_ident<>'hydra' then smc_cmd_v2(cur_vel,'speed',Caxis,0)
 else smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
end;

procedure Tf_speedmode.btn_spd_posMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if ctrl_ident<>'hydra' then smc_cmd_v2('','stopspeed',Caxis,0) else smc_cmd_v2(#3,'',0,0);
end;

procedure Tf_speedmode.btn_spd_dwnClick(Sender: TObject);
begin
if spdcur[CAxis] >0 then dec (spdcur[CAxis]);
 show_vel;
 if spdmode then smc_cmd_v2(cur_vel,'speed',Caxis,0)
 else if spdmode then smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
end;

procedure Tf_speedmode.btn_spd_upClick(Sender: TObject);
begin
 if spdcur[CAxis] <spdstps[CAxis] then inc (spdcur[CAxis]);
 show_vel;
 if spdmode and (ctrl_ident<>'hydra') then smc_cmd_v2(cur_vel,'speed',Caxis,0)
 else if spdmode then smc_cmd_v2(trgt+' '+cur_vel+' '+cur_acc+' '+cur_acc,'pm',Caxis,0);
end;

procedure Tf_speedmode.show_vel;
var speed:double;
begin
  if spdcur[CAxis]> 0 then speed:= spd_max*spdcur[CAxis]/spdstps[CAxis]*spd_dir else speed:=0.0001*spd_dir;
  if (spd_max=0) or (spdcur[CAxis]=0) then lb_vel.Color:=clGray else lb_vel.Color:=clLime;
  lb_vel.Caption:='Speed = '+FormatFloat('0.000000',abs(speed)*sc_fac[CAxis])+actunit[CAxis]+'/s';
  if ctrl_ident<>'hydra' then cur_vel:=FormatFloat('0.000000',speed) else cur_vel:=FormatFloat('0.000000',abs(speed));
  cur_acc:=FormatFloat('0.000000',abs(speed)*10);
end;

procedure Tf_speedmode.get_lmt;
var s,t:string;
    err:integer;
    dmy:double;
begin
 lmt_p:='0';
 lmt_n:='0'; trgt:='';
 tmr_exec:=false;
 s:=trim (smc_cmd_v2('','getnlimit',Caxis,1));
 t:= copy (s,1,Pos(' ',s)-1);
 delete (s,1,pos (' ',s));
 val (t,dmy,err);
 if err=0 then lmt_n:=FormatFloat('0.000000',dmy);
 val (s,dmy,err);
 if err=0 then lmt_p:=FormatFloat('0.000000',dmy);
 tmr_exec:=true;
end;


initialization
  {$I u_speedmode.lrs}

end.

